/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JAxisServlet.java,v 1.5 2005/01/20 14:00:48 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.ws.axis;

import java.io.IOException;
import java.net.URL;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.axis.AxisFault;
import org.apache.axis.server.AxisServer;
import org.apache.axis.transport.http.AxisServlet;
import org.apache.axis.wsdl.gen.Parser;
import org.apache.axis.wsdl.symbolTable.SymbolTable;

import org.objectweb.jonas_ws.deployment.api.ServiceDesc;

/**
 * JOnAS AxisServlet version (add multiple AxisEngine in ServletContext).
 * @author Guillaume Sauthier
 * @deprecated in JOnAS 4.3.0 (since Axis add this feature in their codebase)
 */
public class JAxisServlet extends AxisServlet {

    /**
     * Process a POST to the servlet by handing it off to the Axis Engine.
     * Here is where SOAP messages are received
     * @param req posted request
     * @param res respose
     * @throws ServletException trouble
     * @throws IOException different trouble
     */
    public void doPost(HttpServletRequest req, HttpServletResponse res) throws
    ServletException, IOException {

        // try to see if we already have the parsed WSDL
        ServletContext servletContext = getServletContext();
        Object o = servletContext.getAttribute(getEngine().getName() + "_symbol.table");
        if (o != null) {
            super.doPost(req, res);
            return;
        }


        // else, need to parse WSDL

        // Before calling  super method, add the WSDL in ServletContext
        InitialContext iCtx;
        AxisServer axisServer = getEngine();
        ServiceDesc serviceDesc = null;
        try {
            iCtx = new InitialContext();
            serviceDesc = (ServiceDesc) iCtx.lookup("java:comp/jonas/" + axisServer.getName() + "/dd");
        } catch (NamingException e) {
            throw new AxisFault("Servlet name not found : " + axisServer.getName(), e);
        }

        // Get WSDL
        URL wsdlUrl = serviceDesc.getLocalWSDLURL();

        // Then parse it
        log.debug("WSDL Url found is '" + wsdlUrl + "'");
        SymbolTable symbolTable = null;
        if (wsdlUrl != null) {
            Parser parser = new Parser();
            try {
                //parser.setNowrap(true);
                parser.run(wsdlUrl.toExternalForm());
            } catch (Exception e) {
                log.warn("Cannot get parser on Wsdl file '" + wsdlUrl + "'.");
            }
            if (parser != null) {
                symbolTable = parser.getSymbolTable();
            }
        }


        // Get servlet context
        if (symbolTable != null) {
            if (servletContext != null) {
                servletContext.setAttribute(getEngine().getName() + "_symbol.table", symbolTable);
            }
        }


        super.doPost(req, res);
    }

}