/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.objectweb.jonas_ws.wsgen.generator.ews.wsdltoj2ee.writer;

import java.io.IOException;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.w3c.dom.Node;

import org.apache.axis.wsdl.gen.Generator;
import org.apache.axis.wsdl.symbolTable.SchemaUtils;
import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.apache.axis.wsdl.symbolTable.Type;
import org.apache.axis.wsdl.symbolTable.TypeEntry;
import org.apache.axis.wsdl.toJava.Emitter;
import org.apache.axis.wsdl.toJava.JavaGeneratorFactory;
import org.apache.axis.wsdl.toJava.JavaTypeWriter;
import org.apache.axis.wsdl.toJava.Utils;

/**
 * This is Wsdl2java's Type Writer.  It writes the following files, as appropriate:
 * <typeName>.java, <typeName>Holder.java.
 */
public class JOnASTypeWriter extends JavaTypeWriter implements Generator {

    /** Field typeWriter */
    private Generator jonasTypeWriter = null;

    /**
     * Constructor.
     * 
     * @param emitter     
     * @param type        
     * @param symbolTable 
     */
    public JOnASTypeWriter(Emitter emitter, TypeEntry type,
                          SymbolTable symbolTable) {
        super(emitter,type,symbolTable);
        if (type.isReferenced() && !type.isOnlyLiteralReferenced() && type instanceof Type) {

            // Determine what sort of type this is and instantiate
            // the appropriate Writer.
            Node node = type.getNode();

            boolean isSimpleList = SchemaUtils.isListWithItemType(node);
            // If it's an array, don't emit a class
            if (!type.getName().endsWith("[]") && !isSimpleList) {
                
                // Generate the proper class for either "complex" or "enumeration" types
                Vector v = Utils.getEnumerationBaseAndValues(node, symbolTable);

                if (v != null) {
                    //typeWriter = getEnumTypeWriter(emitter, type, v);
                } else {
                    TypeEntry base =
                            SchemaUtils.getComplexElementExtensionBase(node,
                                    symbolTable);

                    if (base == null) {
                        base = SchemaUtils.getComplexElementRestrictionBase(
                                node, symbolTable);
                    }

                    if (base == null) {
                        QName baseQName = SchemaUtils.getSimpleTypeBase(node);

                        if (baseQName != null) {
                            base = symbolTable.getType(baseQName);
                        }
                    }
                    Vector elements = type.getContainedElements();
                    Vector attributes = type.getContainedAttributes();
                    
                    // If this complexType is referenced in a
                    // fault context, emit a bean-like exception
                    // class
                    Boolean isComplexFault = (Boolean) type.getDynamicVar(
                            JavaGeneratorFactory.COMPLEX_TYPE_FAULT);

                    if ((isComplexFault != null) && isComplexFault.booleanValue()) {

                        jonasTypeWriter = getBeanHelperWriter(emitter, type, elements, base, attributes, true);
                    } else {
                        jonasTypeWriter = getBeanHelperWriter(emitter, type, elements, base, attributes, false) ;
                    }
                }
            }
        }
    }    // ctor

    /**
     * Write all the service bindnigs:  service and testcase.
     * 
     * @throws IOException 
     */
    public void generate() throws IOException {

        if (jonasTypeWriter != null) {
            jonasTypeWriter.generate();
        }
    }    
 
}    // class JavaTypeWriter
